
package org.mule.module.fws.config.spring;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.module.fws.config.PutInboundShipmentMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

public class PutInboundShipmentDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public PutInboundShipmentDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(PutInboundShipmentMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        if ((element.getAttribute("shipmentId")!= null)&&(!StringUtils.isBlank(element.getAttribute("shipmentId")))) {
            builder.addPropertyValue("shipmentId", element.getAttribute("shipmentId"));
        }
        if ((element.getAttribute("shipmentName")!= null)&&(!StringUtils.isBlank(element.getAttribute("shipmentName")))) {
            builder.addPropertyValue("shipmentName", element.getAttribute("shipmentName"));
        }
        if ((element.getAttribute("destinationFulfillmentCenter")!= null)&&(!StringUtils.isBlank(element.getAttribute("destinationFulfillmentCenter")))) {
            builder.addPropertyValue("destinationFulfillmentCenter", element.getAttribute("destinationFulfillmentCenter"));
        }
        if ((element.getAttribute("shipFromAddress-ref")!= null)&&(!StringUtils.isBlank(element.getAttribute("shipFromAddress-ref")))) {
            if (element.getAttribute("shipFromAddress-ref").startsWith("#")) {
                builder.addPropertyValue("shipFromAddress", element.getAttribute("shipFromAddress-ref"));
            } else {
                builder.addPropertyValue("shipFromAddress", (("#[registry:"+ element.getAttribute("shipFromAddress-ref"))+"]"));
            }
        }
        if (element.hasAttribute("labelPreference")) {
            builder.addPropertyValue("labelPreference", element.getAttribute("labelPreference"));
        }
        Element itemQuantitiesListElement = null;
        itemQuantitiesListElement = DomUtils.getChildElementByTagName(element, "item-quantities");
        List<Element> itemQuantitiesListChilds = null;
        if (itemQuantitiesListElement!= null) {
            String itemQuantitiesRef = itemQuantitiesListElement.getAttribute("ref");
            if ((itemQuantitiesRef!= null)&&(!StringUtils.isBlank(itemQuantitiesRef))) {
                if ((!itemQuantitiesRef.startsWith(patternInfo.getPrefix()))&&(!itemQuantitiesRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("itemQuantities", new RuntimeBeanReference(itemQuantitiesRef));
                } else {
                    builder.addPropertyValue("itemQuantities", itemQuantitiesRef);
                }
            } else {
                ManagedList itemQuantities = new ManagedList();
                itemQuantitiesListChilds = DomUtils.getChildElementsByTagName(itemQuantitiesListElement, "item-quantity");
                if (itemQuantitiesListChilds!= null) {
                    for (Element itemQuantitiesChild: itemQuantitiesListChilds) {
                        String valueRef = itemQuantitiesChild.getAttribute("value-ref");
                        if ((valueRef!= null)&&(!StringUtils.isBlank(valueRef))) {
                            itemQuantities.add(new RuntimeBeanReference(valueRef));
                        } else {
                            itemQuantities.add(itemQuantitiesChild.getTextContent());
                        }
                    }
                }
                builder.addPropertyValue("itemQuantities", itemQuantities);
            }
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
            if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                propertyValues.addPropertyValue("messageProcessors", new ManagedList());
            }
            List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
            listMessageProcessors.add(definition);
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
